@@ -1,3 +1,6 @@
+import gc
+import platform
+
 import numpy as np
 import pytest
 
@@ -194,7 +197,8 @@ def test_constrained_layout9():
     fig.suptitle('Test Suptitle', fontsize=28)
 
 
-@image_comparison(['constrained_layout10.png'])
+@image_comparison(['constrained_layout10.png'],
+                  tol=0 if platform.machine() == 'x86_64' else 0.032)
 def test_constrained_layout10():
     """Test for handling legend outside axis"""
     fig, axs = plt.subplots(2, 2, layout="constrained")
@@ -346,7 +350,7 @@ def test_constrained_layout19():
 
 
 def test_constrained_layout20():
-    """Smoke test cl does not mess up added axes"""
+    """Smoke test cl does not mess up added Axes"""
     gx = np.linspace(-5, 5, 4)
     img = np.hypot(gx, gx[:, None])
 
@@ -395,7 +399,7 @@ def test_constrained_layout23():
         fig = plt.figure(layout="constrained", clear=True, num="123")
         gs = fig.add_gridspec(1, 2)
         sub = gs[0].subgridspec(2, 2)
-        fig.suptitle("Suptitle{}".format(i))
+        fig.suptitle(f"Suptitle{i}")
 
 
 @image_comparison(['test_colorbar_location.png'],
@@ -658,6 +662,30 @@ def test_compressed1():
     np.testing.assert_allclose(pos.y0, 0.1934, atol=2e-3)
 
 
+def test_compressed_suptitle():
+    fig, (ax0, ax1) = plt.subplots(
+        nrows=2, figsize=(4, 10), layout="compressed",
+        gridspec_kw={"height_ratios": (1 / 4, 3 / 4), "hspace": 0})
+
+    ax0.axis("equal")
+    ax0.set_box_aspect(1/3)
+
+    ax1.axis("equal")
+    ax1.set_box_aspect(1)
+
+    title = fig.suptitle("Title")
+    fig.draw_without_rendering()
+    assert title.get_position()[1] == pytest.approx(0.7457, abs=1e-3)
+
+    title = fig.suptitle("Title", y=0.98)
+    fig.draw_without_rendering()
+    assert title.get_position()[1] == 0.98
+
+    title = fig.suptitle("Title", in_layout=False)
+    fig.draw_without_rendering()
+    assert title.get_position()[1] == 0.98
+
+
 @pytest.mark.parametrize('arg, state', [
     (True, True),
     (False, False),
@@ -678,3 +706,16 @@ def test_constrained_toggle():
         assert not fig.get_constrained_layout()
         fig.set_constrained_layout(True)
         assert fig.get_constrained_layout()
+
+
+def test_layout_leak():
+    # Make sure there aren't any cyclic references when using LayoutGrid
+    # GH #25853
+    fig = plt.figure(constrained_layout=True, figsize=(10, 10))
+    fig.add_subplot()
+    fig.draw_without_rendering()
+    plt.close("all")
+    del fig
+    gc.collect()
+    assert not any(isinstance(obj, mpl._layoutgrid.LayoutGrid)
+                   for obj in gc.get_objects())
